#!/bin/bash
set -e

function get_containing_directory {
    file_path=$1

    if [ -z ${file_path} ]; then
        echo ""
    else
        file_start_index=$(echo ${file_path} | awk -F"/" '{print length($0)-length($NF)}')
        dir_end_index=$((${file_start_index} - 1))
        dir_path=$(echo ${file_path} | cut -c1-${dir_end_index})
        echo ${dir_path}
    fi
}

function compare_inc_queue_and_core_pattern_directories {
    # if the incoming queue directory does not end with a forward slash then use as it is
    if [[ "$1" == */ ]]; then
        incoming_queue_dir=$(get_containing_directory $1)
    else
        incoming_queue_dir=$1
    fi
    core_pattern_containing_dir=$(get_containing_directory $2)

    # if any of the containing directories are empty return false
    if [ -z ${incoming_queue_dir} ] || [ -z ${core_pattern_containing_dir} ]; then
        return 1
    fi

    if [ ${incoming_queue_dir} == ${core_pattern_containing_dir} ]; then
        return 0
    else
        return 1
    fi
}

function install_watson {
    INC_QUEUE=$1
    REP_QUEUE=$2
    WA_CORE_PATTERN=$3

    use_configured_core_pattern=0
    if [ ! -z ${WA_CORE_PATTERN} ]; then
        if compare_inc_queue_and_core_pattern_directories ${INC_QUEUE} ${WA_CORE_PATTERN}; then
            echo "Incoming queue: ${INC_QUEUE} and Configured core pattern: ${WA_CORE_PATTERN} have same containing directories"
        else
            echo "Warning! Incoming queue: ${INC_QUEUE} and Configured core pattern: ${WA_CORE_PATTERN} have different containing directories"
            use_configured_core_pattern=1
        fi
    else
        echo "Configured core pattern is empty"
        use_configured_core_pattern=1
    fi

    echo 'Create parent folders'
    mkdir -p ${REP_QUEUE}
    mkdir -p ${INC_QUEUE}

    echo 'Setup core collection on crash'
    ulimit -c unlimited

    if [ ${use_configured_core_pattern} -eq 1 ]; then
      DEF_CORE_PATTERN="${INC_QUEUE}/%e.%p"
      echo "Using default core pattern : ${DEF_CORE_PATTERN}"
      sysctl -w kernel.core_pattern="${DEF_CORE_PATTERN}"
    else
      echo "Setting core pattern to : ${WA_CORE_PATTERN}"
      sysctl -w kernel.core_pattern="${WA_CORE_PATTERN}"
    fi

    echo 'Set queue permissions'
    chmod 777 ${INC_QUEUE}
    chmod 555 ${REP_QUEUE}

    echo 'Install completed.'
}

function setup_wa_logs_dir {
  WA_LOGS_DIR=$1
  echo "Creating watsonagent logs directory"
  mkdir -p ${WA_LOGS_DIR} # Create AW agent Logs directory
  # Setting read and execute permission so that others can read the log files from the directory
  chmod 775 ${WA_LOGS_DIR}
}

# Get config values set by user
userconfigfilepath=/opt/msft/watson/userconfig.conf
if [[ -f "$userconfigfilepath" ]]
then
    echo "Importing configuration specified by user"
    source $userconfigfilepath
    rm $userconfigfilepath
fi

# Capture incoming_queue environment variable passed in to the container
if [[ -z "${IncomingPath}" ]]; then
  echo "The environment variable INCOMING_QUEUE is undefined"
  exit 1
else
  INC_QUEUE="${IncomingPath}"
fi

# Capture report_queue environment variable passed in to the container
if [[ -z "${QueuePath}" ]]; then
  echo "The environment variable REPORT_QUEUE is undefined"
  exit 1
else
  REP_QUEUE="${QueuePath}"
fi

# Capture os_release_info_dir environment variable passed in to the container
if [[ -z "${OsReleaseInfoDir}" ]]; then
  echo "The environment variable OS_RELEASE_INFO_DIR is undefined"
fi

# Capture os_subrelease_info_dir environment variable passed in to the container
if [[ -z "${OsSubreleaseInfoDir}" ]]; then
  echo "The environment variable OS_SUBRELEASE_INFO_DIR is undefined"
fi

#Capture container_name_separator  environment variable used to separate container name appended to dumpfile
if [[ -z "${ContainerNameSeparator}" ]]; then
  echo "The environment variable CONTAINER_NAME_SEPARATOR is undefined"
fi


if [[ -z "${AwLogsDirectory}" ]]; then
  echo "The environment variable WA_LOGS_DIR is undefined"
else
  WA_LOGS_DIR="${AwLogsDirectory}"WA_LOGS_DIR
  setup_wa_logs_dir ${AwLogsDirectory}
fi

if [[ -z "${WaCorePattern}" ]]; then
  echo "The environment variable WA_CORE_PATTERN is undefined"
else
  WA_CORE_PATTERN="${WaCorePattern}"
fi

install_watson ${INC_QUEUE} ${REP_QUEUE} ${WA_CORE_PATTERN}
MACHINENAME=
  if [ -f /bin/hostname ]; then
    MACHINENAME=`/bin/hostname`
fi

MACHINEID=
if [ -f /etc/machine-id ]; then
    MACHINEID=`cat /etc/machine-id`
fi

if [ ! -z "${MACHINENAME}" ]; then
    export MACHINENAME
fi

if [ ! -z "${MACHINEID}" ]; then
    export MACHINEID
fi

IS_MULTILOG_INSTALLED=$(echo $(command -v multilog))

# Start running watsonagent
if [ ! -z "${IS_MULTILOG_INSTALLED}" ] && [ ! -z "${WA_LOGS_DIR}" ]; then
    echo "Daemontools is installed , Logs directory: ${WA_LOGS_DIR}"
    # Redirecting stdout of watsonagent process to the console as well as multilog
    watsonagent --configuration=/etc/watsonagent.conf --verbose | tee >(multilog s10000 n7 ${WA_LOGS_DIR})
else
    echo "Daemontools is not installed or Logs directory not found"
    watsonagent --configuration=/etc/watsonagent.conf
fi
